@@ -15,8 +15,7 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   see <http://www.gnu.org/licenses/>.
 
    Author: Stef Walter <nielsen@memberwebs.com>
 */
@@ -28,11 +27,12 @@
 #include <glib-object.h>
 #include <gio/gio.h>
 
-#include "pkcs11.h"
+#include <p11-kit/pkcs11.h>
 
 #define __GCK_INSIDE_HEADER__
 
 #include <gck/gck-enum-types.h>
+#include <gck/gck-version.h>
 
 G_BEGIN_DECLS
 
@@ -41,9 +41,11 @@ G_BEGIN_DECLS
  * and add the C flag: -DGCK_API_SUBJECT_TO_CHANGE
  */
 
+#ifndef __GI_SCANNER__
 #ifndef GCK_API_SUBJECT_TO_CHANGE
 #error "This API has not yet reached stability."
 #endif
+#endif
 
 #define             GCK_VENDOR_CODE                         0x47434B00 /* GCK */
 
@@ -98,7 +100,7 @@ struct _GckAttribute {
 	gulong length;
 };
 
-#define GCK_INVALID G_MAXULONG
+#define GCK_INVALID ((gulong)-1)
 
 gboolean            gck_value_to_ulong                      (const guchar *value,
                                                              gsize length,
@@ -173,6 +175,9 @@ gchar*              gck_attribute_get_string                (const GckAttribute
 void                gck_attribute_get_date                  (const GckAttribute *attr,
                                                              GDate* value);
 
+const guchar *      gck_attribute_get_data                  (const GckAttribute *attr,
+                                                             gsize *length);
+
 gboolean            gck_attribute_equal                     (gconstpointer attr1,
                                                              gconstpointer attr2);
 
@@ -186,6 +191,8 @@ void                gck_attribute_free                      (gpointer attr);
 
 void                gck_attribute_dump                      (const GckAttribute *attr);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckAttribute, gck_attribute_free);
+
 typedef struct _GckBuilder GckBuilder;
 
 struct _GckBuilder {
@@ -332,7 +339,9 @@ GckBuilder *         gck_builder_copy                       (GckBuilder *builder
 
 void                 gck_builder_clear                      (GckBuilder *builder);
 
-#define              GCK_TYPE_ATTRIBUTES                    (gck_attributes_get_boxed_type ())
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckBuilder, gck_builder_unref);
+
+#define              GCK_TYPE_ATTRIBUTES                    (gck_attributes_get_type ())
 
 GType                gck_attributes_get_type                (void) G_GNUC_CONST;
 
@@ -378,6 +387,8 @@ void                 gck_attributes_dump                    (GckAttributes *attr
 
 gchar *              gck_attributes_to_string               (GckAttributes *attrs);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckAttributes, gck_attributes_unref);
+
 /* -------------------------------------------------------------------------
  * FORWARDS
  */
@@ -415,6 +426,8 @@ GckModuleInfo *     gck_module_info_copy                   (GckModuleInfo *modul
 
 void                gck_module_info_free                   (GckModuleInfo *module_info);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckModuleInfo, gck_module_info_free);
+
 #define GCK_TYPE_MODULE             (gck_module_get_type())
 #define GCK_MODULE(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), GCK_TYPE_MODULE, GckModule))
 #define GCK_MODULE_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass), GCK_TYPE_MODULE, GckModule))
@@ -517,6 +530,8 @@ GckEnumerator*        gck_modules_enumerate_uri               (GList *modules,
                                                                GckSessionOptions session_options,
                                                                GError **error);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckModule, g_object_unref);
+
 
 /* ------------------------------------------------------------------------
  * ENUMERATOR
@@ -559,6 +574,11 @@ GType                 gck_enumerator_get_object_type          (GckEnumerator *se
 void                  gck_enumerator_set_object_type          (GckEnumerator *self,
                                                                GType object_type);
 
+void                  gck_enumerator_set_object_type_full     (GckEnumerator *self,
+                                                               GType object_type,
+                                                               const gulong *attr_types,
+                                                               gint attr_count);
+
 GckEnumerator *       gck_enumerator_get_chained              (GckEnumerator *self);
 
 void                  gck_enumerator_set_chained              (GckEnumerator *self,
@@ -583,6 +603,8 @@ GList*                gck_enumerator_next_finish              (GckEnumerator *se
                                                                GAsyncResult *result,
                                                                GError **error);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckEnumerator, g_object_unref);
+
 /* ------------------------------------------------------------------------
  * SLOT
  */
@@ -607,6 +629,8 @@ GckSlotInfo *       gck_slot_info_copy                      (GckSlotInfo *slot_i
 
 void                gck_slot_info_free                      (GckSlotInfo *slot_info);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckSlotInfo, gck_slot_info_free);
+
 typedef struct _GckTokenInfo GckTokenInfo;
 
 struct _GckTokenInfo {
@@ -640,6 +664,8 @@ GckTokenInfo *      gck_token_info_copy                     (GckTokenInfo *token
 
 void                gck_token_info_free                     (GckTokenInfo *token_info);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckTokenInfo, gck_token_info_free);
+
 typedef struct _GckMechanismInfo GckMechanismInfo;
 
 struct _GckMechanismInfo {
@@ -663,6 +689,8 @@ void                gck_mechanism_info_free                 (GckMechanismInfo *m
 gboolean            gck_mechanisms_check                    (GArray *mechanisms,
                                                              ...);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckMechanismInfo, gck_mechanism_info_free);
+
 #define GCK_TYPE_SLOT             (gck_slot_get_type())
 #define GCK_SLOT(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), GCK_TYPE_SLOT, GckSlot))
 #define GCK_SLOT_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass), GCK_TYPE_SLOT, GckSlot))
@@ -757,6 +785,8 @@ GckEnumerator*      gck_slots_enumerate_objects             (GList *slots,
                                                              GckAttributes *match,
                                                              GckSessionOptions options);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckSlot, g_object_unref);
+
 /* ------------------------------------------------------------------------
  * SESSION
  */
@@ -778,6 +808,8 @@ GckSessionInfo *    gck_session_info_copy                  (GckSessionInfo *sess
 
 void                gck_session_info_free                  (GckSessionInfo *session_info);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckSessionInfo, gck_session_info_free);
+
 #define GCK_TYPE_SESSION             (gck_session_get_type())
 #define GCK_SESSION(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), GCK_TYPE_SESSION, GckSession))
 #define GCK_SESSION_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass), GCK_TYPE_SESSION, GckSession))
@@ -1231,6 +1263,8 @@ GckObject*          gck_session_derive_key_finish            (GckSession *self,
                                                               GAsyncResult *result,
                                                               GError **error);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckSession, g_object_unref);
+
 /* ------------------------------------------------------------------------
  * OBJECT
  */
@@ -1387,6 +1421,8 @@ GckAttributes*      gck_object_get_template_finish          (GckObject *self,
                                                              GAsyncResult *result,
                                                              GError **error);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckObject, g_object_unref);
+
 /* ------------------------------------------------------------------------
  * OBJECT ATTRIBUTES
  */
@@ -1455,6 +1491,8 @@ GckAttributes *     gck_object_cache_lookup_finish         (GckObject *object,
                                                             GAsyncResult *result,
                                                             GError **error);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckObjectCache, g_object_unref);
+
 /* ------------------------------------------------------------------------
  * PASSWORD
  */
@@ -1493,6 +1531,8 @@ GckSlot *           gck_password_get_token                  (GckPassword *self);
 
 GckObject *         gck_password_get_key                    (GckPassword *self);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckPassword, g_object_unref);
+
 /* ----------------------------------------------------------------------------
  * URI
  */
@@ -1551,6 +1591,8 @@ GckUriData *        gck_uri_data_copy                       (GckUriData *uri_dat
 
 void                gck_uri_data_free                       (GckUriData *uri_data);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GckUriData, gck_uri_data_free);
+
 G_END_DECLS
 
 #include "gck-deprecated.h"
